@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -230,8 +230,7 @@ PROPERTY_SIMPLE_CASE_DESCRIPTION = """
     Given a subclass of HasTraits with a trait that is defined as Property
     that depends on a simple trait, compare having the Property be defined
     as Property(), Property(depends_on="name"), Property(observe="name"),
-    and Property(observe=trait('name')). (Note that this is a feature
-    currently available on master only).
+    and Property(observe=trait('name')).
 
     The timing we are interested in:
     (1) Time to import the module, i.e. the class construction.
@@ -277,7 +276,7 @@ def get_property_simple_timing(property_args, cached_property):
     """ Time the cases described by the (cached) property depending on a simple
     trait scenario.  Whether or not the property is cached is based on the
     cached_property argument, and the given property_args argument is used in
-    the Property trait defintion.
+    the Property trait definition.
 
     Parameters
     ----------
@@ -326,8 +325,7 @@ PROPERTY_EXTENDED_CASE_DESCRIPTION = """
     that depends on an extended trait, compare having the Property be
     defined as Property(), Property(depends_on="child.name"),
     Property(observe="child.name"), and
-    Property(observe=trait('child').trait('name')). (Note that this is a
-    feature currently available on master only).
+    Property(observe=trait('child').trait('name')).
 
     The timing we are interested in:
     (1) Time to import the module, i.e. the class construction.
@@ -375,7 +373,7 @@ def get_property_extended_timing(property_args, cached_property):
     """ Time the cases described by the (cached) property depending on an
     extended trait scenario. Wether or not the property is cached is determined
     by the cached_property argument, and the given property_args argument is
-    used for the Property trait defintion.
+    used for the Property trait definition.
 
     Parameters
     ----------
