@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -25,16 +25,76 @@ from traits.api import (
     Int,
     List,
     observe,
+    Property,
     Set,
     Str,
+    Undefined,
 )
 from traits.observation.api import (
+    anytrait,
     pop_exception_handler,
     push_exception_handler,
     trait,
 )
 
 
+class TestObserveDecorator(unittest.TestCase):
+    """ General tests for the observe decorator. """
+
+    def test_warning_on_handler_with_bad_signature(self):
+        message_regex = "should be callable with a single positional argument"
+
+        with self.assertWarnsRegex(UserWarning, message_regex):
+            class A(HasTraits):
+                foo = Int()
+
+                @observe("foo")
+                def _do_something_when_foo_changes(self):
+                    pass
+
+        with self.assertWarnsRegex(UserWarning, message_regex):
+            class B(HasTraits):
+                foo = Int()
+
+                @observe("foo")
+                def _do_something_when_foo_changes(self, **kwargs):
+                    pass
+
+    def test_decorated_method_signatures(self):
+        # Test different handler signatures for compatibility with
+        # observe decorator.
+
+        class A(HasTraits):
+            foo = Int()
+
+            call_count = Int(0)
+
+            @observe("foo")
+            def _the_usual_signature(self, event):
+                self.call_count += 1
+
+            @observe("foo")
+            def _method_with_extra_optional_args(self, event, frombicate=True):
+                self.call_count += 1
+
+            @observe("foo")
+            def _method_with_star_args(self, *args):
+                self.call_count += 1
+
+            @observe("foo")
+            def _method_with_alternative_name(self, foo_change_event):
+                self.call_count += 1
+
+            @observe("foo")
+            def _optional_second_argument(self, event=None):
+                self.call_count += 1
+
+        a = A()
+        self.assertEqual(a.call_count, 0)
+        a.foo = 23
+        self.assertEqual(a.call_count, 5)
+
+
 class Student(HasTraits):
     """ Model for testing list + post_init (enthought/traits#275) """
 
@@ -675,3 +735,174 @@ class TestObserveItemsFromOnTraitChange(unittest.TestCase):
         app.trait_property_changed("i_am_undefined_with_items", 1, 2)
 
         self.assertEqual(len(events), 1)
+
+
+# Integration tests for anytrait listener -------------------------------------
+
+class HasVariousTraits(HasTraits):
+    #: Function to call on any trait change.
+    trait_change_callback = Any()
+
+    foo = Int(16)
+
+    bar = Str("off")
+
+    updated = Event(Bool)
+
+    @observe("*")
+    def _record_trait_change(self, event):
+        callback = self.trait_change_callback
+        if callback is not None:
+            callback(event)
+
+
+class UpdateListener(HasTraits):
+    foo = Instance(HasVariousTraits)
+
+    bar = Instance(HasVariousTraits)
+
+
+class TestObserveAnytrait(unittest.TestCase):
+    def test_observe_method_anytrait(self):
+        obj = HasVariousTraits()
+
+        events = []
+        obj.observe(events.append, "*")
+        obj.foo = 23
+        obj.bar = "on"
+
+        self.assertEqual(len(events), 2)
+        foo_event, bar_event = events
+
+        self.assertEqual(foo_event.object, obj)
+        self.assertEqual(foo_event.name, "foo")
+        self.assertEqual(foo_event.old, 16)
+        self.assertEqual(foo_event.new, 23)
+
+        self.assertEqual(bar_event.object, obj)
+        self.assertEqual(bar_event.name, "bar")
+        self.assertEqual(bar_event.old, "off")
+        self.assertEqual(bar_event.new, "on")
+
+    def test_observe_decorator_anytrait(self):
+        events = []
+        obj = HasVariousTraits(trait_change_callback=events.append)
+        obj.foo = 23
+        obj.bar = "on"
+
+        self.assertEqual(len(events), 3)
+        callback_event, foo_event, bar_event = events
+
+        self.assertEqual(callback_event.object, obj)
+        self.assertEqual(callback_event.name, "trait_change_callback")
+        self.assertIs(callback_event.old, None)
+        self.assertEqual(callback_event.new, events.append)
+
+        self.assertEqual(foo_event.object, obj)
+        self.assertEqual(foo_event.name, "foo")
+        self.assertEqual(foo_event.old, 16)
+        self.assertEqual(foo_event.new, 23)
+
+        self.assertEqual(bar_event.object, obj)
+        self.assertEqual(bar_event.name, "bar")
+        self.assertEqual(bar_event.old, "off")
+        self.assertEqual(bar_event.new, "on")
+
+    def test_anytrait_expression(self):
+        obj = HasVariousTraits()
+
+        events = []
+        obj.observe(events.append, anytrait())
+        obj.foo = 23
+        obj.bar = "on"
+
+        self.assertEqual(len(events), 2)
+        foo_event, bar_event = events
+
+        self.assertEqual(foo_event.object, obj)
+        self.assertEqual(foo_event.name, "foo")
+        self.assertEqual(foo_event.old, 16)
+        self.assertEqual(foo_event.new, 23)
+
+        self.assertEqual(bar_event.object, obj)
+        self.assertEqual(bar_event.name, "bar")
+        self.assertEqual(bar_event.old, "off")
+        self.assertEqual(bar_event.new, "on")
+
+    def test_anytrait_method(self):
+        foo = HasVariousTraits()
+        bar = HasVariousTraits()
+        obj = UpdateListener(foo=foo, bar=bar)
+
+        events = []
+        obj.observe(events.append, trait("foo", notify=False).anytrait())
+
+        foo.updated = True
+        bar.updated = True
+
+        self.assertEqual(len(events), 1)
+        foo_event, = events
+
+        self.assertEqual(foo_event.object, foo)
+        self.assertEqual(foo_event.name, "updated")
+        self.assertEqual(foo_event.old, Undefined)
+        self.assertEqual(foo_event.new, True)
+
+    def test_anytrait_with_children(self):
+        foo = HasVariousTraits()
+        bar = HasVariousTraits()
+        obj = UpdateListener(foo=foo, bar=bar)
+
+        events = []
+        with self.assertRaises(ValueError):
+            obj.observe(events.append, "*:updated")
+
+    def test_anytrait_of_anytrait(self):
+        foo = HasVariousTraits()
+        bar = HasVariousTraits()
+        obj = UpdateListener(foo=foo, bar=bar)
+
+        events = []
+        with self.assertRaises(ValueError):
+            obj.observe(events.append, "*:*")
+
+    def test_anytrait_unobserve(self):
+        obj = HasVariousTraits()
+
+        events = []
+        obj.observe(events.append, "*")
+        obj.foo = 23
+        obj.bar = "on"
+
+        self.assertEqual(len(events), 2)
+
+        obj.observe(events.append, "*", remove=True)
+
+        obj.foo = 232
+        obj.bar = "mid"
+
+        # No additional events.
+        self.assertEqual(len(events), 2)
+
+    def test_property_subclass_observe(self):
+        # Regression test for enthought/traits#1586
+        class Base(HasTraits):
+            bar = Int()
+
+            foo = Property(Int(), observe="bar")
+
+            def _get_foo(self):
+                return self.bar
+
+        class Derived(Base):
+            pass
+
+        events = []
+
+        obj = Derived(bar=3)
+        obj.observe(events.append, "foo")
+
+        # Changing bar should result in a single event.
+        self.assertEqual(len(events), 0)
+        obj.bar = 5
+        self.assertEqual(len(events), 1)
